@@ -1,5 +1,6 @@
 //! Main SVG document structure.
 
+use data_url::mime::Mime;
 use gdk_pixbuf::{PixbufLoader, PixbufLoaderExt};
 use markup5ever::QualName;
 use once_cell::sync::Lazy;
@@ -9,6 +10,7 @@ use std::collections::HashMap;
 use std::fmt;
 use std::include_str;
 use std::rc::Rc;
+use std::str::FromStr;
 
 use crate::css::{self, Origin, Stylesheet};
 use crate::error::{AcquireError, AllowedUrlError, LoadingError, NodeIdError};
@@ -104,6 +106,22 @@ impl Document {
         )
     }
 
+    /// Utility function to load a document from a static string in tests.
+    #[cfg(test)]
+    pub fn load_from_bytes(input: &'static [u8]) -> Document {
+        use glib::prelude::*;
+
+        let bytes = glib::Bytes::from_static(input);
+        let stream = gio::MemoryInputStream::from_bytes(&bytes);
+
+        Document::load_from_stream(
+            &LoadOptions::new(UrlResolver::new(None)),
+            &stream.upcast(),
+            None::<&gio::Cancellable>,
+        )
+        .unwrap()
+    }
+
     /// Gets the root node.  This is guaranteed to be an `<svg>` element.
     pub fn root(&self) -> Node {
         self.tree.clone()
@@ -232,24 +250,17 @@ fn load_image(
 ) -> Result<SharedImageSurface, LoadingError> {
     let BinaryData {
         data: bytes,
-        mut content_type,
+        mime_type,
     } = io::acquire_data(&aurl, None)?;
 
     if bytes.is_empty() {
         return Err(LoadingError::Other(String::from("no image data")));
     }
 
-    // See issue #548 - data: URLs without a MIME-type automatically
-    // fall back to "text/plain;charset=US-ASCII".  Some (old?) versions of
-    // Adobe Illustrator generate data: URLs without MIME-type for image
-    // data.  We'll catch this and fall back to sniffing by unsetting the
-    // content_type.
-    if content_type.as_deref() == Some("text/plain;charset=US-ASCII") {
-        content_type = None;
-    }
+    let content_type = content_type_for_gdk_pixbuf(&mime_type);
 
     let loader = if let Some(ref content_type) = content_type {
-        PixbufLoader::new_with_mime_type(content_type)?
+        PixbufLoader::with_mime_type(content_type)?
     } else {
         PixbufLoader::new()
     };
@@ -273,6 +284,21 @@ fn load_image(
     Ok(surface)
 }
 
+fn content_type_for_gdk_pixbuf(mime_type: &Mime) -> Option<String> {
+    // See issue #548 - data: URLs without a MIME-type automatically
+    // fall back to "text/plain;charset=US-ASCII".  Some (old?) versions of
+    // Adobe Illustrator generate data: URLs without MIME-type for image
+    // data.  We'll catch this and fall back to sniffing by unsetting the
+    // content_type.
+    let unspecified_mime_type = Mime::from_str("text/plain;charset=US-ASCII").unwrap();
+
+    if *mime_type == unspecified_mime_type {
+        None
+    } else {
+        Some(format!("{}/{}", mime_type.type_, mime_type.subtype))
+    }
+}
+
 fn human_readable_url(aurl: &AllowedUrl) -> &str {
     if aurl.scheme() == "data" {
         // avoid printing a huge data: URL for image data
@@ -282,12 +308,12 @@ fn human_readable_url(aurl: &AllowedUrl) -> &str {
     }
 }
 
-fn image_loading_error_from_cairo(status: cairo::Status, aurl: &AllowedUrl) -> LoadingError {
+fn image_loading_error_from_cairo(status: cairo::Error, aurl: &AllowedUrl) -> LoadingError {
     let url = human_readable_url(aurl);
 
     match status {
-        cairo::Status::NoMemory => LoadingError::OutOfMemory(format!("loading image: {}", url)),
-        cairo::Status::InvalidSize => LoadingError::Other(format!("image too big: {}", url)),
+        cairo::Error::NoMemory => LoadingError::OutOfMemory(format!("loading image: {}", url)),
+        cairo::Error::InvalidSize => LoadingError::Other(format!("image too big: {}", url)),
         _ => LoadingError::Other(format!("cairo error: {}", status)),
     }
 }
@@ -564,4 +590,21 @@ mod tests {
             Err(NodeIdError::NodeIdRequired)
         ));
     }
+
+    #[test]
+    fn unspecified_mime_type_yields_no_content_type() {
+        // Issue #548
+        let mime = Mime::from_str("text/plain;charset=US-ASCII").unwrap();
+        assert!(content_type_for_gdk_pixbuf(&mime).is_none());
+    }
+
+    #[test]
+    fn strips_mime_type_parameters() {
+        // Issue #699
+        let mime = Mime::from_str("image/png;charset=utf-8").unwrap();
+        assert_eq!(
+            content_type_for_gdk_pixbuf(&mime),
+            Some(String::from("image/png"))
+        );
+    }
 }
