@@ -1,16 +1,38 @@
-use cssparser::Parser;
+//! SVG2 filter function shortcuts - `blur()`, `brightness()`, etc.
+//!
+//! The `<filter>` element from SVG1.1 (also present in SVG2) uses some verbose XML to
+//! define chains of filter primitives.  In SVG2, there is a shortcut form of the `filter`
+//! attribute and property, where one can simply say `filter="blur(5)"` and get the
+//! equivalent of writing a full `<filter>` with a `<feGaussianBlur>` element.
+//!
+//! This module has a type for each of the filter functions in SVG2 with the function's
+//! parameters, for example [`Blur`](struct.Blur.html) stores the blur's standard
+//! deviation parameter.
+//!
+//! Those types get aggregated in the [`FilterFunction`](enum.FilterFunction.html) enum.
+//! A `FilterFunction` can then convert itself into a
+//! [`FilterSpec`](../filters/struct.FilterSpec.html), which is ready to be rendered on a
+//! surface.
 
+use cssparser::{Color, Parser, RGBA};
+
+use crate::angle::Angle;
 use crate::error::*;
 use crate::filter::Filter;
 use crate::filters::{
-    gaussian_blur::GaussianBlur, FilterResolveError, FilterSpec, Primitive, PrimitiveParams,
-    ResolvedPrimitive,
+    color_matrix::ColorMatrix,
+    component_transfer,
+    composite::{Composite, Operator},
+    flood::Flood,
+    gaussian_blur::GaussianBlur,
+    merge::{Merge, MergeNode},
+    offset::Offset,
+    FilterResolveError, FilterSpec, Input, Primitive, PrimitiveParams, ResolvedPrimitive,
 };
 use crate::length::*;
-use crate::parsers::{NumberOrPercentage, Parse};
-use crate::properties::ComputedValues;
-use crate::{coord_units::CoordUnits, filters::color_matrix::ColorMatrix};
-use crate::{drawing_ctx::DrawingCtx, filters::component_transfer};
+use crate::paint_server::resolve_color;
+use crate::parsers::{CustomIdent, NumberOrPercentage, Parse};
+use crate::unit_interval::UnitInterval;
 
 /// CSS Filter functions from the Filter Effects Module Level 1
 ///
@@ -20,7 +42,9 @@ pub enum FilterFunction {
     Blur(Blur),
     Brightness(Brightness),
     Contrast(Contrast),
+    DropShadow(DropShadow),
     Grayscale(Grayscale),
+    HueRotate(HueRotate),
     Invert(Invert),
     Opacity(Opacity),
     Saturate(Saturate),
@@ -51,6 +75,17 @@ pub struct Contrast {
     proportion: Option<f64>,
 }
 
+/// Parameters for the `drop-shadow()` filter function
+///
+/// https://www.w3.org/TR/filter-effects/#funcdef-filter-drop-shadow
+#[derive(Debug, Clone, PartialEq)]
+pub struct DropShadow {
+    color: Option<Color>,
+    dx: Option<Length<Horizontal>>,
+    dy: Option<Length<Vertical>>,
+    std_deviation: Option<ULength<Both>>,
+}
+
 /// Parameters for the `grayscale()` filter function
 ///
 /// https://www.w3.org/TR/filter-effects/#funcdef-filter-grayscale
@@ -59,6 +94,14 @@ pub struct Grayscale {
     proportion: Option<f64>,
 }
 
+/// Parameters for the `hue-rotate()` filter function
+///
+/// https://www.w3.org/TR/filter-effects/#funcdef-filter-huerotate
+#[derive(Debug, Clone, PartialEq)]
+pub struct HueRotate {
+    angle: Option<Angle>,
+}
+
 /// Parameters for the `invert()` filter function
 ///
 /// https://www.w3.org/TR/filter-effects/#funcdef-filter-invert
@@ -143,6 +186,41 @@ fn parse_contrast<'i>(parser: &mut Parser<'i, '_>) -> Result<FilterFunction, Par
 
     Ok(FilterFunction::Contrast(Contrast { proportion }))
 }
+#[allow(clippy::unnecessary_wraps)]
+fn parse_dropshadow<'i>(parser: &mut Parser<'i, '_>) -> Result<FilterFunction, ParseError<'i>> {
+    let mut result = DropShadow {
+        color: None,
+        dx: None,
+        dy: None,
+        std_deviation: None,
+    };
+
+    result.color = parser.try_parse(|p| Color::parse(p)).ok();
+
+    // if dx is provided, dy must follow and an optional std_dev must follow that.
+    if let Ok(dx) = parser.try_parse(|p| Length::parse(p)) {
+        result.dx = Some(dx);
+        result.dy = Some(parser.try_parse(|p| Length::parse(p))?);
+        result.std_deviation = parser.try_parse(|p| ULength::parse(p)).ok();
+    }
+
+    let loc = parser.current_source_location();
+
+    // because the color and length arguments can be provided in either order,
+    // check again after potentially parsing lengths if the color is now provided.
+    // if a color is provided both before and after, that is an error.
+    if let Ok(c) = parser.try_parse(|p| Color::parse(p)) {
+        if result.color.is_some() {
+            return Err(
+                loc.new_custom_error(ValueErrorKind::Value("color already specified".to_string()))
+            );
+        } else {
+            result.color = Some(c);
+        }
+    }
+
+    Ok(FilterFunction::DropShadow(result))
+}
 
 #[allow(clippy::unnecessary_wraps)]
 fn parse_grayscale<'i>(parser: &mut Parser<'i, '_>) -> Result<FilterFunction, ParseError<'i>> {
@@ -151,6 +229,13 @@ fn parse_grayscale<'i>(parser: &mut Parser<'i, '_>) -> Result<FilterFunction, Pa
     Ok(FilterFunction::Grayscale(Grayscale { proportion }))
 }
 
+#[allow(clippy::unnecessary_wraps)]
+fn parse_huerotate<'i>(parser: &mut Parser<'i, '_>) -> Result<FilterFunction, ParseError<'i>> {
+    let angle = parser.try_parse(|p| Angle::parse(p)).ok();
+
+    Ok(FilterFunction::HueRotate(HueRotate { angle }))
+}
+
 #[allow(clippy::unnecessary_wraps)]
 fn parse_invert<'i>(parser: &mut Parser<'i, '_>) -> Result<FilterFunction, ParseError<'i>> {
     let proportion = parse_num_or_percentage_clamped(parser);
@@ -282,6 +367,83 @@ impl Contrast {
     }
 }
 
+impl DropShadow {
+    /// Converts a DropShadow into the set of filter element primitives.
+    ///
+    /// See https://www.w3.org/TR/filter-effects/#dropshadowEquivalent
+    fn to_filter_spec(&self, params: &NormalizeParams, default_color: RGBA) -> FilterSpec {
+        let user_space_filter = Filter::default().to_user_space(params);
+        let dx = self.dx.map(|l| l.to_user(params)).unwrap_or(0.0);
+        let dy = self.dy.map(|l| l.to_user(params)).unwrap_or(0.0);
+        let std_dev = self.std_deviation.map(|l| l.to_user(params)).unwrap_or(0.0);
+        let color = self
+            .color
+            .as_ref()
+            .map(|c| resolve_color(c, UnitInterval::clamp(1.0), default_color))
+            .unwrap_or(default_color);
+
+        let offsetblur = CustomIdent("offsetblur".to_string());
+
+        let gaussian_blur = ResolvedPrimitive {
+            primitive: Primitive::default(),
+            params: PrimitiveParams::GaussianBlur(GaussianBlur {
+                in1: Input::SourceAlpha,
+                std_deviation: (std_dev, std_dev),
+                ..GaussianBlur::default()
+            }),
+        }
+        .into_user_space(params);
+
+        let offset = ResolvedPrimitive {
+            primitive: Primitive {
+                result: Some(offsetblur.clone()),
+                ..Primitive::default()
+            },
+            params: PrimitiveParams::Offset(Offset {
+                in1: Input::default(),
+                dx,
+                dy,
+            }),
+        }
+        .into_user_space(params);
+
+        let flood = ResolvedPrimitive {
+            primitive: Primitive::default(),
+            params: PrimitiveParams::Flood(Flood { color }),
+        }
+        .into_user_space(params);
+
+        let composite = ResolvedPrimitive {
+            primitive: Primitive::default(),
+            params: PrimitiveParams::Composite(Composite {
+                in2: Input::FilterOutput(offsetblur),
+                operator: Operator::In,
+                ..Composite::default()
+            }),
+        }
+        .into_user_space(params);
+
+        let merge = ResolvedPrimitive {
+            primitive: Primitive::default(),
+            params: PrimitiveParams::Merge(Merge {
+                merge_nodes: vec![
+                    MergeNode::default(),
+                    MergeNode {
+                        in1: Input::SourceGraphic,
+                        ..MergeNode::default()
+                    },
+                ],
+            }),
+        }
+        .into_user_space(params);
+
+        FilterSpec {
+            user_space_filter,
+            primitives: vec![gaussian_blur, offset, flood, composite, merge],
+        }
+    }
+}
+
 impl Grayscale {
     fn to_filter_spec(&self, params: &NormalizeParams) -> FilterSpec {
         // grayscale is implemented as the inverse of a saturate operation,
@@ -295,6 +457,27 @@ impl Grayscale {
     }
 }
 
+impl HueRotate {
+    fn to_filter_spec(&self, params: &NormalizeParams) -> FilterSpec {
+        let rads = self.angle.map(|a| a.radians()).unwrap_or(0.0);
+        let user_space_filter = Filter::default().to_user_space(params);
+
+        let huerotate = ResolvedPrimitive {
+            primitive: Primitive::default(),
+            params: PrimitiveParams::ColorMatrix(ColorMatrix {
+                matrix: ColorMatrix::hue_rotate_matrix(rads),
+                ..ColorMatrix::default()
+            }),
+        }
+        .into_user_space(params);
+
+        FilterSpec {
+            user_space_filter,
+            primitives: vec![huerotate],
+        }
+    }
+}
+
 impl Invert {
     fn to_filter_spec(&self, params: &NormalizeParams) -> FilterSpec {
         let p = self.proportion.unwrap_or(1.0);
@@ -434,7 +617,9 @@ impl Parse for FilterFunction {
             ("blur", &parse_blur),
             ("brightness", &parse_brightness),
             ("contrast", &parse_contrast),
+            ("drop-shadow", &parse_dropshadow),
             ("grayscale", &parse_grayscale),
+            ("hue-rotate", &parse_huerotate),
             ("invert", &parse_invert),
             ("opacity", &parse_opacity),
             ("saturate", &parse_saturate),
@@ -456,18 +641,16 @@ impl FilterFunction {
     #[allow(clippy::unnecessary_wraps)]
     pub fn to_filter_spec(
         &self,
-        values: &ComputedValues,
-        draw_ctx: &DrawingCtx,
+        params: &NormalizeParams,
+        current_color: RGBA,
     ) -> Result<FilterSpec, FilterResolveError> {
-        // userSpaceonUse is the default for primitive_units
-        let view_params = draw_ctx.push_coord_units(CoordUnits::UserSpaceOnUse);
-        let params = NormalizeParams::new(values, &view_params);
-
         match self {
             FilterFunction::Blur(v) => Ok(v.to_filter_spec(&params)),
             FilterFunction::Brightness(v) => Ok(v.to_filter_spec(&params)),
             FilterFunction::Contrast(v) => Ok(v.to_filter_spec(&params)),
+            FilterFunction::DropShadow(v) => Ok(v.to_filter_spec(&params, current_color)),
             FilterFunction::Grayscale(v) => Ok(v.to_filter_spec(&params)),
+            FilterFunction::HueRotate(v) => Ok(v.to_filter_spec(&params)),
             FilterFunction::Invert(v) => Ok(v.to_filter_spec(&params)),
             FilterFunction::Opacity(v) => Ok(v.to_filter_spec(&params)),
             FilterFunction::Saturate(v) => Ok(v.to_filter_spec(&params)),
@@ -527,6 +710,69 @@ mod tests {
         );
     }
 
+    #[test]
+    fn parses_dropshadow() {
+        assert_eq!(
+            FilterFunction::parse_str("drop-shadow(4px 5px)").unwrap(),
+            FilterFunction::DropShadow(DropShadow {
+                color: None,
+                dx: Some(Length::new(4.0, LengthUnit::Px)),
+                dy: Some(Length::new(5.0, LengthUnit::Px)),
+                std_deviation: None,
+            })
+        );
+
+        assert_eq!(
+            FilterFunction::parse_str("drop-shadow(#ff0000 4px 5px 32px)").unwrap(),
+            FilterFunction::DropShadow(DropShadow {
+                color: Some(Color::RGBA(RGBA {
+                    red: 255,
+                    green: 0,
+                    blue: 0,
+                    alpha: 255
+                })),
+                dx: Some(Length::new(4.0, LengthUnit::Px)),
+                dy: Some(Length::new(5.0, LengthUnit::Px)),
+                std_deviation: Some(ULength::new(32.0, LengthUnit::Px)),
+            })
+        );
+
+        assert_eq!(
+            FilterFunction::parse_str("drop-shadow(1px 2px blue)").unwrap(),
+            FilterFunction::DropShadow(DropShadow {
+                color: Some(Color::RGBA(RGBA {
+                    red: 0,
+                    green: 0,
+                    blue: 255,
+                    alpha: 255
+                })),
+                dx: Some(Length::new(1.0, LengthUnit::Px)),
+                dy: Some(Length::new(2.0, LengthUnit::Px)),
+                std_deviation: None,
+            })
+        );
+
+        assert_eq!(
+            FilterFunction::parse_str("drop-shadow(1px 2px 3px currentColor)").unwrap(),
+            FilterFunction::DropShadow(DropShadow {
+                color: Some(Color::CurrentColor),
+                dx: Some(Length::new(1.0, LengthUnit::Px)),
+                dy: Some(Length::new(2.0, LengthUnit::Px)),
+                std_deviation: Some(ULength::new(3.0, LengthUnit::Px)),
+            })
+        );
+
+        assert_eq!(
+            FilterFunction::parse_str("drop-shadow(1 2 3)").unwrap(),
+            FilterFunction::DropShadow(DropShadow {
+                color: None,
+                dx: Some(Length::new(1.0, LengthUnit::Px)),
+                dy: Some(Length::new(2.0, LengthUnit::Px)),
+                std_deviation: Some(ULength::new(3.0, LengthUnit::Px)),
+            })
+        );
+    }
+
     #[test]
     fn parses_grayscale() {
         assert_eq!(
@@ -542,6 +788,28 @@ mod tests {
         );
     }
 
+    #[test]
+    fn parses_huerotate() {
+        assert_eq!(
+            FilterFunction::parse_str("hue-rotate()").unwrap(),
+            FilterFunction::HueRotate(HueRotate { angle: None })
+        );
+
+        assert_eq!(
+            FilterFunction::parse_str("hue-rotate(0)").unwrap(),
+            FilterFunction::HueRotate(HueRotate {
+                angle: Some(Angle::new(0.0))
+            })
+        );
+
+        assert_eq!(
+            FilterFunction::parse_str("hue-rotate(128deg)").unwrap(),
+            FilterFunction::HueRotate(HueRotate {
+                angle: Some(Angle::from_degrees(128.0))
+            })
+        );
+    }
+
     #[test]
     fn parses_invert() {
         assert_eq!(
@@ -639,11 +907,24 @@ mod tests {
         assert!(FilterFunction::parse_str("contrast(foo)").is_err());
     }
 
+    #[test]
+    fn invalid_dropshadow_yields_error() {
+        assert!(FilterFunction::parse_str("drop-shadow(blue 5px green)").is_err());
+        assert!(FilterFunction::parse_str("drop-shadow(blue 5px 5px green)").is_err());
+        assert!(FilterFunction::parse_str("drop-shadow(blue 1px)").is_err());
+        assert!(FilterFunction::parse_str("drop-shadow(1 2 3 4 blue)").is_err());
+    }
+
     #[test]
     fn invalid_grayscale_yields_error() {
         assert!(FilterFunction::parse_str("grayscale(foo)").is_err());
     }
 
+    #[test]
+    fn invalid_huerotate_yields_error() {
+        assert!(FilterFunction::parse_str("hue-rotate(foo)").is_err());
+    }
+
     #[test]
     fn invalid_invert_yields_error() {
         assert!(FilterFunction::parse_str("invert(foo)").is_err());
