@@ -4,11 +4,9 @@ use cssparser::Parser;
 
 use crate::bbox::BoundingBox;
 use crate::document::{AcquiredNodes, NodeId};
-use crate::drawing_ctx::DrawingCtx;
+use crate::drawing_ctx::ViewParams;
 use crate::element::Element;
-use crate::error::{
-    AcquireError, ImplementationLimit, NodeIdError, ParseError, RenderingError, ValueErrorKind,
-};
+use crate::error::{AcquireError, NodeIdError, ParseError, ValueErrorKind};
 use crate::gradient::{ResolvedGradient, UserSpaceGradient};
 use crate::node::NodeBorrow;
 use crate::parsers::Parse;
@@ -17,16 +15,33 @@ use crate::properties::ComputedValues;
 use crate::unit_interval::UnitInterval;
 use crate::util;
 
+/// Unresolved SVG paint server straight from the DOM data.
+///
+/// This is either a solid color (which if `currentColor` needs to be extracted from the
+/// `ComputedValues`), or a paint server like a gradient or pattern which is referenced by
+/// a URL that points to a certain document node.
+///
+/// Use [`PaintServer.resolve`](#method.resolve) to turn this into a
+/// [`PaintSource`](enum.PaintSource.html).
 #[derive(Debug, Clone, PartialEq)]
 pub enum PaintServer {
+    /// For example, `fill="none"`.
     None,
+
+    /// For example, `fill="url(#some_gradient) fallback_color"`.
     Iri {
         iri: Box<NodeId>,
         alternate: Option<cssparser::Color>,
     },
+
+    /// For example, `fill="blue"`.
     SolidColor(cssparser::Color),
 }
 
+/// Paint server with resolved references, with unnormalized lengths.
+///
+/// Use [`PaintSource.to_user_space`](#method.to_user_space) to turn this into a
+/// [`UserSpacePaintSource`](enum.UserSpacePaintSource.html).
 pub enum PaintSource {
     None,
     Gradient(ResolvedGradient, Option<cssparser::RGBA>),
@@ -34,6 +49,9 @@ pub enum PaintSource {
     SolidColor(cssparser::RGBA),
 }
 
+/// Fully resolved paint server, in user-space units.
+///
+/// This has everything required for rendering.
 pub enum UserSpacePaintSource {
     None,
     Gradient(UserSpaceGradient, Option<cssparser::RGBA>),
@@ -79,12 +97,18 @@ impl Parse for PaintServer {
 }
 
 impl PaintServer {
+    /// Resolves colors, plus node references for gradients and patterns.
+    ///
+    /// `opacity` depends on `strokeOpacity` or `fillOpacity` depending on whether
+    /// the paint server is for the `stroke` or `fill` properties.
+    ///
+    /// `current_color` should be the value of `ComputedValues.color()`.
     pub fn resolve(
         &self,
         acquired_nodes: &mut AcquiredNodes<'_>,
         opacity: UnitInterval,
         current_color: cssparser::RGBA,
-    ) -> Result<PaintSource, RenderingError> {
+    ) -> PaintSource {
         match self {
             PaintServer::Iri {
                 ref iri,
@@ -123,79 +147,82 @@ impl PaintServer {
                         _ => Err(AcquireError::InvalidLinkType(iri.as_ref().clone())),
                     }
                 })
-                .or_else(|err| match (err, alternate) {
-                    (AcquireError::MaxReferencesExceeded, _) => {
-                        rsvg_log!("exceeded maximum number of referenced objects");
-                        Err(RenderingError::LimitExceeded(
-                            ImplementationLimit::TooManyReferencedElements,
-                        ))
-                    }
-
-                    // The following two cases catch AcquireError::CircularReference, which for
-                    // paint servers may mean that there is a pattern or gradient with a reference
-                    // cycle in its "href" attribute.  This is an invalid paint server, and per
-                    // https://www.w3.org/TR/SVG2/painting.html#SpecifyingPaint we should try to
-                    // fall back to the alternate color.
-                    (_, Some(color)) => {
+                .unwrap_or_else(|_| match alternate {
+                    // The following cases catch AcquireError::CircularReference and
+                    // AcquireError::MaxReferencesExceeded.
+                    //
+                    // Circular references mean that there is a pattern or gradient with a
+                    // reference cycle in its "href" attribute.  This is an invalid paint
+                    // server, and per
+                    // https://www.w3.org/TR/SVG2/painting.html#SpecifyingPaint we should
+                    // try to fall back to the alternate color.
+                    //
+                    // Exceeding the maximum number of references will get caught again
+                    // later in the drawing code, so it should be fine to translate this
+                    // condition to that for an invalid paint server.
+                    Some(color) => {
                         rsvg_log!(
                             "could not resolve paint server \"{}\", using alternate color",
                             iri
                         );
 
-                        Ok(PaintSource::SolidColor(resolve_color(
-                            color,
-                            opacity,
-                            current_color,
-                        )))
+                        PaintSource::SolidColor(resolve_color(color, opacity, current_color))
                     }
 
-                    (_, _) => {
+                    None => {
                         rsvg_log!(
                             "could not resolve paint server \"{}\", no alternate color specified",
                             iri
                         );
 
-                        Ok(PaintSource::None)
+                        PaintSource::None
                     }
                 }),
 
-            PaintServer::SolidColor(color) => Ok(PaintSource::SolidColor(resolve_color(
-                color,
-                opacity,
-                current_color,
-            ))),
+            PaintServer::SolidColor(color) => {
+                PaintSource::SolidColor(resolve_color(color, opacity, current_color))
+            }
 
-            PaintServer::None => Ok(PaintSource::None),
+            PaintServer::None => PaintSource::None,
         }
     }
 }
 
 impl PaintSource {
+    /// Converts lengths to user-space.
     pub fn to_user_space(
         &self,
         bbox: &BoundingBox,
-        draw_ctx: &DrawingCtx,
+        current_params: &ViewParams,
         values: &ComputedValues,
     ) -> UserSpacePaintSource {
         match *self {
             PaintSource::None => UserSpacePaintSource::None,
             PaintSource::SolidColor(c) => UserSpacePaintSource::SolidColor(c),
 
-            PaintSource::Gradient(ref g, c) => match (g.to_user_space(bbox, draw_ctx, values), c) {
-                (Some(gradient), c) => UserSpacePaintSource::Gradient(gradient, c),
-                (None, Some(c)) => UserSpacePaintSource::SolidColor(c),
-                (None, None) => UserSpacePaintSource::None,
-            },
+            PaintSource::Gradient(ref g, c) => {
+                match (g.to_user_space(bbox, current_params, values), c) {
+                    (Some(gradient), c) => UserSpacePaintSource::Gradient(gradient, c),
+                    (None, Some(c)) => UserSpacePaintSource::SolidColor(c),
+                    (None, None) => UserSpacePaintSource::None,
+                }
+            }
 
-            PaintSource::Pattern(ref p, c) => match (p.to_user_space(bbox, draw_ctx, values), c) {
-                (Some(pattern), c) => UserSpacePaintSource::Pattern(pattern, c),
-                (None, Some(c)) => UserSpacePaintSource::SolidColor(c),
-                (None, None) => UserSpacePaintSource::None,
-            },
+            PaintSource::Pattern(ref p, c) => {
+                match (p.to_user_space(bbox, current_params, values), c) {
+                    (Some(pattern), c) => UserSpacePaintSource::Pattern(pattern, c),
+                    (None, Some(c)) => UserSpacePaintSource::SolidColor(c),
+                    (None, None) => UserSpacePaintSource::None,
+                }
+            }
         }
     }
 }
 
+/// Resolves a CSS color into an RGBA value.
+///
+/// A CSS color can be `currentColor`, in which case the computed value comes from
+/// the `color` property.  You should pass the `color` property's value for `current_color`.
 pub fn resolve_color(
     color: &cssparser::Color,
     opacity: UnitInterval,
