@@ -11,32 +11,33 @@ use crate::document::AcquiredNodes;
 use crate::drawing_ctx::DrawingCtx;
 use crate::element::{Draw, ElementResult, SetAttributes};
 use crate::error::*;
+use crate::iri::Iri;
+use crate::layout::{Shape, StackingContext, Stroke};
 use crate::length::*;
-use crate::node::{CascadedValues, Node};
+use crate::node::{CascadedValues, Node, NodeBorrow};
 use crate::parsers::{optional_comma, Parse, ParseValue};
 use crate::path_builder::{LargeArc, Path as SvgPath, PathBuilder, Sweep};
-use crate::path_parser;
 use crate::xml::Attributes;
 
-#[derive(Copy, Clone, PartialEq)]
-pub enum Markers {
+#[derive(PartialEq)]
+enum Markers {
     No,
     Yes,
 }
 
-pub struct Shape {
-    pub path: Rc<SvgPath>,
-    pub markers: Markers,
+struct ShapeDef {
+    path: Rc<SvgPath>,
+    markers: Markers,
 }
 
-impl Shape {
-    fn new(path: Rc<SvgPath>, markers: Markers) -> Shape {
-        Shape { path, markers }
+impl ShapeDef {
+    fn new(path: Rc<SvgPath>, markers: Markers) -> ShapeDef {
+        ShapeDef { path, markers }
     }
 }
 
 trait BasicShape {
-    fn make_shape(&self, params: &NormalizeParams) -> Shape;
+    fn make_shape(&self, params: &NormalizeParams) -> ShapeDef;
 }
 
 macro_rules! impl_draw {
@@ -53,13 +54,96 @@ macro_rules! impl_draw {
                 let values = cascaded.get();
                 let view_params = draw_ctx.get_view_params();
                 let params = NormalizeParams::new(values, &view_params);
-                let shape = self.make_shape(&params);
-                draw_ctx.draw_shape(&shape, node, acquired_nodes, values, clipping)
+                let shape_def = self.make_shape(&params);
+
+                let is_visible = values.is_visible();
+                let paint_order = values.paint_order();
+
+                let stroke = Stroke::new(values, &params);
+
+                let stroke_paint = values.stroke().0.resolve(
+                    acquired_nodes,
+                    values.stroke_opacity().0,
+                    values.color().0,
+                );
+
+                let fill_paint = values.fill().0.resolve(
+                    acquired_nodes,
+                    values.fill_opacity().0,
+                    values.color().0,
+                );
+
+                let fill_rule = values.fill_rule();
+                let clip_rule = values.clip_rule();
+                let shape_rendering = values.shape_rendering();
+
+                let marker_start;
+                let marker_mid;
+                let marker_end;
+
+                if shape_def.markers == Markers::Yes {
+                    marker_start = acquire_marker(acquired_nodes, &values.marker_start().0);
+                    marker_mid = acquire_marker(acquired_nodes, &values.marker_mid().0);
+                    marker_end = acquire_marker(acquired_nodes, &values.marker_end().0);
+                } else {
+                    marker_start = None;
+                    marker_mid = None;
+                    marker_end = None;
+                }
+
+                let shape = Shape {
+                    path: shape_def.path,
+                    is_visible,
+                    paint_order,
+                    stroke,
+                    stroke_paint,
+                    fill_paint,
+                    fill_rule,
+                    clip_rule,
+                    shape_rendering,
+                    marker_start,
+                    marker_mid,
+                    marker_end,
+                };
+
+                let elt = node.borrow_element();
+                let stacking_ctx =
+                    StackingContext::new(acquired_nodes, &elt, elt.get_transform(), values);
+
+                draw_ctx.draw_shape(
+                    &view_params,
+                    &shape,
+                    &stacking_ctx,
+                    acquired_nodes,
+                    values,
+                    clipping,
+                )
             }
         }
     };
 }
 
+fn acquire_marker(acquired_nodes: &mut AcquiredNodes<'_>, iri: &Iri) -> Option<Node> {
+    iri.get().and_then(|id| {
+        acquired_nodes
+            .acquire(id)
+            .map_err(|e| {
+                rsvg_log!("cannot render marker: {}", e);
+            })
+            .ok()
+            .and_then(|acquired| {
+                let node = acquired.get();
+
+                if is_element_of_type!(node, Marker) {
+                    Some(node.clone())
+                } else {
+                    rsvg_log!("{} is not a marker element", id);
+                    None
+                }
+            })
+    })
+}
+
 fn make_ellipse(cx: f64, cy: f64, rx: f64, ry: f64) -> SvgPath {
     let mut builder = PathBuilder::default();
 
@@ -128,7 +212,7 @@ impl SetAttributes for Path {
         for (attr, value) in attrs.iter() {
             if attr.expanded() == expanded_name!("", "d") {
                 let mut builder = PathBuilder::default();
-                if let Err(e) = path_parser::parse_path_into_builder(value, &mut builder) {
+                if let Err(e) = builder.parse(value) {
                     // FIXME: we don't propagate errors upstream, but creating a partial
                     // path is OK per the spec
 
@@ -143,8 +227,8 @@ impl SetAttributes for Path {
 }
 
 impl BasicShape for Path {
-    fn make_shape(&self, _params: &NormalizeParams) -> Shape {
-        Shape::new(self.path.clone(), Markers::Yes)
+    fn make_shape(&self, _params: &NormalizeParams) -> ShapeDef {
+        ShapeDef::new(self.path.clone(), Markers::Yes)
     }
 }
 
@@ -224,8 +308,8 @@ impl SetAttributes for Polygon {
 }
 
 impl BasicShape for Polygon {
-    fn make_shape(&self, _params: &NormalizeParams) -> Shape {
-        Shape::new(Rc::new(make_poly(&self.points, true)), Markers::Yes)
+    fn make_shape(&self, _params: &NormalizeParams) -> ShapeDef {
+        ShapeDef::new(Rc::new(make_poly(&self.points, true)), Markers::Yes)
     }
 }
 
@@ -249,8 +333,8 @@ impl SetAttributes for Polyline {
 }
 
 impl BasicShape for Polyline {
-    fn make_shape(&self, _params: &NormalizeParams) -> Shape {
-        Shape::new(Rc::new(make_poly(&self.points, false)), Markers::Yes)
+    fn make_shape(&self, _params: &NormalizeParams) -> ShapeDef {
+        ShapeDef::new(Rc::new(make_poly(&self.points, false)), Markers::Yes)
     }
 }
 
@@ -281,7 +365,7 @@ impl SetAttributes for Line {
 }
 
 impl BasicShape for Line {
-    fn make_shape(&self, params: &NormalizeParams) -> Shape {
+    fn make_shape(&self, params: &NormalizeParams) -> ShapeDef {
         let mut builder = PathBuilder::default();
 
         let x1 = self.x1.to_user(params);
@@ -292,7 +376,7 @@ impl BasicShape for Line {
         builder.move_to(x1, y1);
         builder.line_to(x2, y2);
 
-        Shape::new(Rc::new(builder.into_path()), Markers::Yes)
+        ShapeDef::new(Rc::new(builder.into_path()), Markers::Yes)
     }
 }
 
@@ -330,7 +414,7 @@ impl SetAttributes for Rect {
 
 impl BasicShape for Rect {
     #[allow(clippy::many_single_char_names)]
-    fn make_shape(&self, params: &NormalizeParams) -> Shape {
+    fn make_shape(&self, params: &NormalizeParams) -> ShapeDef {
         let x = self.x.to_user(params);
         let y = self.y.to_user(params);
         let w = self.width.to_user(params);
@@ -379,7 +463,7 @@ impl BasicShape for Rect {
 
         // Per the spec, w,h must be >= 0
         if w <= 0.0 || h <= 0.0 {
-            return Shape::new(Rc::new(builder.into_path()), Markers::No);
+            return ShapeDef::new(Rc::new(builder.into_path()), Markers::No);
         }
 
         let half_w = w / 2.0;
@@ -505,7 +589,7 @@ impl BasicShape for Rect {
             builder.close_path();
         }
 
-        Shape::new(Rc::new(builder.into_path()), Markers::No)
+        ShapeDef::new(Rc::new(builder.into_path()), Markers::No)
     }
 }
 
@@ -534,12 +618,12 @@ impl SetAttributes for Circle {
 }
 
 impl BasicShape for Circle {
-    fn make_shape(&self, params: &NormalizeParams) -> Shape {
+    fn make_shape(&self, params: &NormalizeParams) -> ShapeDef {
         let cx = self.cx.to_user(params);
         let cy = self.cy.to_user(params);
         let r = self.r.to_user(params);
 
-        Shape::new(Rc::new(make_ellipse(cx, cy, r, r)), Markers::No)
+        ShapeDef::new(Rc::new(make_ellipse(cx, cy, r, r)), Markers::No)
     }
 }
 
@@ -570,13 +654,13 @@ impl SetAttributes for Ellipse {
 }
 
 impl BasicShape for Ellipse {
-    fn make_shape(&self, params: &NormalizeParams) -> Shape {
+    fn make_shape(&self, params: &NormalizeParams) -> ShapeDef {
         let cx = self.cx.to_user(params);
         let cy = self.cy.to_user(params);
         let rx = self.rx.to_user(params);
         let ry = self.ry.to_user(params);
 
-        Shape::new(Rc::new(make_ellipse(cx, cy, rx, ry)), Markers::No)
+        ShapeDef::new(Rc::new(make_ellipse(cx, cy, rx, ry)), Markers::No)
     }
 }
 
