@@ -51,6 +51,30 @@ impl SurfaceType {
     }
 }
 
+/// Operators supported by `ImageSurface<Shared>::compose`.
+pub enum Operator {
+    Over,
+    In,
+    Out,
+    Atop,
+    Xor,
+    Multiply,
+    Screen,
+    Darken,
+    Lighten,
+    Overlay,
+    ColorDodge,
+    ColorBurn,
+    HardLight,
+    SoftLight,
+    Difference,
+    Exclusion,
+    HslHue,
+    HslSaturation,
+    HslColor,
+    HslLuminosity,
+}
+
 /// Wrapper for a Cairo image surface that enforces exclusive access when modifying it.
 ///
 /// Shared access to `cairo::ImageSurface` is tricky since a read-only borrowed reference
@@ -1101,7 +1125,7 @@ impl ImageSurface<Shared> {
     }
 
     /// Performs the combination of two input surfaces using Porter-Duff
-    /// compositing operators
+    /// compositing operators.
     ///
     /// # Panics
     /// Panics if the two surface types are not compatible.
@@ -1110,7 +1134,7 @@ impl ImageSurface<Shared> {
         &self,
         other: &SharedImageSurface,
         bounds: IRect,
-        operator: cairo::Operator,
+        operator: Operator,
     ) -> Result<SharedImageSurface, cairo::Error> {
         let output_surface = other.copy_surface(bounds)?;
 
@@ -1121,7 +1145,7 @@ impl ImageSurface<Shared> {
             cr.clip();
 
             self.set_as_source_surface(&cr, 0.0, 0.0);
-            cr.set_operator(operator);
+            cr.set_operator(operator.into());
             cr.paint();
         }
 
@@ -1347,6 +1371,36 @@ impl ImageSurface<Exclusive> {
     }
 }
 
+impl From<Operator> for cairo::Operator {
+    fn from(op: Operator) -> cairo::Operator {
+        use cairo::Operator as Cairo;
+        use Operator::*;
+
+        match op {
+            Over => Cairo::Over,
+            In => Cairo::In,
+            Out => Cairo::Out,
+            Atop => Cairo::Atop,
+            Xor => Cairo::Xor,
+            Multiply => Cairo::Multiply,
+            Screen => Cairo::Screen,
+            Darken => Cairo::Darken,
+            Lighten => Cairo::Lighten,
+            Overlay => Cairo::Overlay,
+            ColorDodge => Cairo::ColorDodge,
+            ColorBurn => Cairo::ColorBurn,
+            HardLight => Cairo::HardLight,
+            SoftLight => Cairo::SoftLight,
+            Difference => Cairo::Difference,
+            Exclusion => Cairo::Exclusion,
+            HslHue => Cairo::HslHue,
+            HslSaturation => Cairo::HslSaturation,
+            HslColor => Cairo::HslColor,
+            HslLuminosity => Cairo::HslLuminosity,
+        }
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
