@@ -529,16 +529,103 @@ fn negative_resolution_is_invalid() {
         .stderr(contains("Invalid resolution"));
 }
 
+#[test]
+fn zero_offset_png() {
+    RsvgConvert::new_with_input("tests/fixtures/cmdline/dimensions-in.svg")
+        .arg("--page-width=640")
+        .arg("--page-height=480")
+        .arg("--width=200")
+        .arg("--height=100")
+        .assert()
+        .success()
+        .stdout(file::is_png().with_contents("tests/fixtures/cmdline/zero-offset-png.png"));
+}
+
+#[test]
+fn offset_png() {
+    RsvgConvert::new_with_input("tests/fixtures/cmdline/dimensions-in.svg")
+        .arg("--page-width=640")
+        .arg("--page-height=480")
+        .arg("--width=200")
+        .arg("--height=100")
+        .arg("--left=100")
+        .arg("--top=50")
+        .assert()
+        .success()
+        .stdout(file::is_png().with_contents("tests/fixtures/cmdline/offset-png.png"));
+}
+
 #[cfg(system_deps_have_cairo_pdf)]
 #[test]
-fn pdf_page_size() {
-    RsvgConvert::new_with_input("tests/fixtures/dimensions/521-with-viewbox.svg")
+fn unscaled_pdf_size() {
+    RsvgConvert::new_with_input("tests/fixtures/cmdline/dimensions-in.svg")
         .arg("--format=pdf")
         .assert()
         .success()
-        // TODO: the PDF size and resolution is actually a bug in rsvg-convert,
-        // see https://gitlab.gnome.org/GNOME/librsvg/issues/514
-        .stdout(file::is_pdf().with_page_size(200, 100, 72.0));
+        .stdout(file::is_pdf().with_page_size(72.0, 72.0));
+}
+
+#[cfg(system_deps_have_cairo_pdf)]
+#[test]
+fn pdf_size_width_height() {
+    RsvgConvert::new_with_input("tests/fixtures/cmdline/dimensions-in.svg")
+        .arg("--format=pdf")
+        .arg("--width=2in")
+        .arg("--height=3in")
+        .assert()
+        .success()
+        .stdout(file::is_pdf().with_page_size(144.0, 216.0));
+}
+
+#[cfg(system_deps_have_cairo_pdf)]
+#[test]
+fn pdf_size_width_height_proportional() {
+    RsvgConvert::new_with_input("tests/fixtures/cmdline/dimensions-in.svg")
+        .arg("--format=pdf")
+        .arg("--width=2in")
+        .arg("--height=3in")
+        .arg("--keep-aspect-ratio")
+        .assert()
+        .success()
+        .stdout(file::is_pdf().with_page_size(144.0, 144.0));
+}
+
+#[cfg(system_deps_have_cairo_pdf)]
+#[test]
+fn pdf_page_size() {
+    RsvgConvert::new_with_input("tests/fixtures/cmdline/dimensions-in.svg")
+        .arg("--format=pdf")
+        .arg("--page-width=210mm")
+        .arg("--page-height=297mm")
+        .assert()
+        .success()
+        .stdout(file::is_pdf().with_page_size(210.0 / 25.4 * 72.0, 297.0 / 25.4 * 72.0));
+}
+
+#[cfg(system_deps_have_cairo_pdf)]
+#[test]
+fn missing_page_size_yields_error() {
+    RsvgConvert::new_with_input("tests/fixtures/cmdline/dimensions-in.svg")
+        .arg("--format=pdf")
+        .arg("--page-width=210mm")
+        .assert()
+        .failure()
+        .stderr(contains("both").and(contains("options")));
+
+    RsvgConvert::new_with_input("tests/fixtures/cmdline/dimensions-in.svg")
+        .arg("--format=pdf")
+        .arg("--page-height=297mm")
+        .assert()
+        .failure()
+        .stderr(contains("both").and(contains("options")));
+}
+
+#[test]
+fn does_not_clip_partial_coverage_pixels() {
+    RsvgConvert::new_with_input("tests/fixtures/cmdline/677-partial-pixel.svg")
+        .assert()
+        .success()
+        .stdout(file::is_png().with_size(2, 2));
 }
 
 #[test]
